#include "stdafx.h"
#include "FormMain.h"
#include <sys/stat.h>

using namespace System;
using namespace System::Windows;
using namespace System::Windows::Forms;
using namespace System::Drawing;

namespace AudioFixLite {

#define COLUMN_POS_STATUS				0
#define COLUMN_POS_GAIN					4
#define COLUMN_POS_MAX_DECIBEL			5
#define COLUMN_POS_MAX_SAMPLE_POS		6
#define COLUMN_POS_LOUDNESS				7
#define COLUMN_POS_PHASE_GAP			8
#define COLUMN_POS_MOVE_PHASE			9
#define COLUMN_POS_CLICK_NOISE_COUNT	10
#define COLUMN_POS_REPAIR_CLICK_NOISE	11
#define COLUMN_POS_AVERAGE				12
#define COLUMN_POS_MOVE_DC_OFFSET		13

ref class MenuItemValue {
public:
	ToolStripMenuItem^ m_item;
	int m_value;
	MenuItemValue(ToolStripMenuItem^ item, int value) {
		m_item = item;
		m_value = value;
	}
};

PointF GetDpiMagnif() {
	HDC screen = GetDC(0);
	double magnifX = GetDeviceCaps(screen, LOGPIXELSX) / 96.0;
	double magnifY = GetDeviceCaps(screen, LOGPIXELSY) / 96.0;
	ReleaseDC(0, screen);
	return PointF(magnifX, magnifY);
}

System::Void FormMain::FormMain_Load(System::Object^  sender, System::EventArgs^  e) {
	m_priorities->Add(gcnew MenuItemValue(highestToolStripMenuItem, THREAD_PRIORITY_HIGHEST));
	m_priorities->Add(gcnew MenuItemValue(highToolStripMenuItem, THREAD_PRIORITY_ABOVE_NORMAL));
	m_priorities->Add(gcnew MenuItemValue(normalToolStripMenuItem, THREAD_PRIORITY_NORMAL));
	m_priorities->Add(gcnew MenuItemValue(lowToolStripMenuItem, THREAD_PRIORITY_BELOW_NORMAL));
	m_priorities->Add(gcnew MenuItemValue(lowestToolStripMenuItem, THREAD_PRIORITY_LOWEST));
	LoadPrefFile();
	FixMenu();
}

bool FormMain::AddWaveFileWithDlalog() {
	openWaveFileDialog->Filter = gcnew String(_T("WAVE/AIFF files|*.wav;*.wave;*.aif;*.aiff|WAVE files|*.wav;*.wave|AIFF files|*.aif;*.aiff|All files (*.*)|*.*"));
	openWaveFileDialog->InitialDirectory = Prefs::data->waveDirPath;
//	openWaveFileDialog->InitialDirectory = gcnew String(Prefs::data->waveDirPath.c_str());
	System::Windows::Forms::DialogResult result;
	result = openWaveFileDialog->ShowDialog();
	if (result != System::Windows::Forms::DialogResult::OK)
		return false;
	int count = openWaveFileDialog->FileNames->Length;
	if (count == 0)
		return false;
	std::list<std::wstring> list;
	FileInfo^ fInfo = gcnew FileInfo((String^)openWaveFileDialog->FileNames[0]);
	Prefs::data->waveDirPath = fInfo->Directory->FullName;
//	Prefs::data->waveDirPath = TStringUtil::MarshalString(fInfo->Directory->FullName);
	for (int i = 0; i < count; i++) {
		list.push_back(TStringUtil::MarshalString((String^)openWaveFileDialog->FileNames[i]));
	}
	list.sort();
	while (list.size() > 0) {
		AddWaveFile(list.front());
		list.pop_front();
	}
	return true;
}

bool FormMain::AddWaveFile(const std::wstring &waveFilePath) {
	if (TAudioFile::CheckExtension(waveFilePath, _T(".wav")) == false &&
		TAudioFile::CheckExtension(waveFilePath, _T(".wave")) == false &&
		TAudioFile::CheckExtension(waveFilePath, _T(".aif")) == false &&
		TAudioFile::CheckExtension(waveFilePath, _T(".aiff")) == false) {
		return false;
	}
	if (GetAudioFileByWaveFilePath(waveFilePath) != NULL)
		return false;

	TAudioFile *af = new TAudioFile;
	if (af->Setup(waveFilePath) == false)
		return false;
	m_files->push_back(af);
	int n = lvFileList->Items->Count;

	TCriticalSectionEnter cse(m_csFileList);

	ListViewItem^ item = gcnew ListViewItem(_T("-"));
	item->SubItems->Add(gcnew String(af->m_artist.c_str()));
	item->SubItems->Add(gcnew String(af->m_album.c_str()));
	item->SubItems->Add(gcnew String(af->GetWaveFileName().c_str()));
	item->SubItems->Add(af->m_gain.ToString());
	item->SubItems->Add(gcnew String(af->GetMaxDecibelString().c_str()));
	item->SubItems->Add(gcnew String(af->GetMaxPosString().c_str()));
	item->SubItems->Add(gcnew String(af->GetMaxLaudDecibelString().c_str()));
	item->SubItems->Add(gcnew String(af->GetPhaseGapAverageString().c_str()));
	item->SubItems->Add(gcnew String(af->GetMovePhaseString().c_str()));
	item->SubItems->Add(gcnew String(af->GetClickNoiseCountString().c_str()));
	item->SubItems->Add(gcnew String(af->GetDeClickNoiseString().c_str()));
	item->SubItems->Add(gcnew String(af->GetAverageString().c_str()));
	item->SubItems->Add(gcnew String(af->GetMoveDCOffsetString().c_str()));
	item->SubItems->Add(gcnew String(af->GetHzString().c_str()));
	item->SubItems->Add(gcnew String(af->GetBitsString().c_str()));
	item->SubItems->Add(gcnew String(af->GetChString().c_str()));
	item->SubItems->Add(gcnew String(af->GetTimeString().c_str()));
	item->SubItems->Add(gcnew String(af->GetMp3FileName().c_str()));
	item->Tag = gcnew TagOfAudioFile(af);
	lvFileList->Items->Add(item);
	return true;
}

TAudioFile *FormMain::GetAudioFileByWaveFilePath(const std::wstring &waveFilePath) {
	for (size_t i = 0; i < m_files->size(); i++) {
		TAudioFile *af = m_files->at(i);
		if (_wcsicmp(af->m_waveFilePath.c_str(), waveFilePath.c_str()) == 0) {
			return af;
		}
	}
	return NULL;
}

ListViewItem^ FormMain::GetListViewItemByAudioFile(TAudioFile *af) {
	int n = lvFileList->Items->Count;
	for (int i = 0; i < n; i++) {
		ListViewItem^ item = lvFileList->Items[i];
		TagOfAudioFile^ tagx = (TagOfAudioFile^)item->Tag;
		if (tagx->m_af == af) {
			return item;
		}
	}
	return nullptr;
}

void FormMain::CallbackEncodeProgress(TGenericCallbackParam^ param) {
	//TCriticalSectionEnter cse(m_csFileList);
	TAudioFile *af = (TAudioFile *)param->GetParam();
	ListViewItem^ item = GetListViewItemByAudioFile(af);
	if (item != nullptr) {
		std::wstring status = af->GetStatusString();
		item->SubItems[COLUMN_POS_STATUS]->Text = gcnew String(status.c_str());
		if (af->m_status >= 0)
			mainToolStripProgressBar->Value = af->m_status;
	}
}

void FormMain::CallbackEncodeFinishOne(TGenericCallbackParam^ param) {
	CallbackEncodeProgress(param);
	mainToolStripProgressBar->Value = 0;
}

void FormMain::CallbackCommonProgress(TGenericCallbackParam^ param) {
	int v = (int)param->GetParam();
	if (v < 0)
		v = 0;
	if (v > 100)
		v = 100;
	mainToolStripProgressBar->Value = v;
}

void FormMain::CallbackCommonStartOne(TGenericCallbackParam^ param) {
	TAudioFile *af = (TAudioFile *)param->GetParam();
	ListViewItem^ item = GetListViewItemByAudioFile(af);
	if (item != nullptr) {
		std::wstring status = af->GetStatusString();
		item->SubItems[COLUMN_POS_STATUS]->Text = gcnew String("*");
	}
}

void FormMain::CallbackCommonFinishOne(TGenericCallbackParam^ param) {
	TAudioFile *af = (TAudioFile *)param->GetParam();
	ListViewItem^ item = GetListViewItemByAudioFile(af);
	if (item != nullptr) {
		item->SubItems[COLUMN_POS_STATUS]->Text = gcnew String("-");
		if (m_files->GetThreadCommand() == THR_CMD_CHECK_MAX_DECIBEL || 
			m_files->GetThreadCommand() == THR_CMD_CHECK_PHASE_GAP) {
			item->SubItems[COLUMN_POS_MAX_DECIBEL]->Text = gcnew String(af->GetMaxDecibelString().c_str());
			item->SubItems[COLUMN_POS_MAX_SAMPLE_POS]->Text = gcnew String(af->GetMaxPosString().c_str());
			item->SubItems[COLUMN_POS_LOUDNESS]->Text = gcnew String(af->GetMaxLaudDecibelString().c_str());
			item->SubItems[COLUMN_POS_PHASE_GAP]->Text = gcnew String(af->GetPhaseGapAverageString().c_str());
			item->SubItems[COLUMN_POS_AVERAGE]->Text = gcnew String(af->GetAverageString().c_str());
			//item->SubItems[COLUMN_POS_DC_OFFSET]->Text = gcnew String(af->GetDcOffsetString().c_str());
		}
		else if (m_files->GetThreadCommand() == THR_CMD_CHECK_CLICK_NOISE) {
			item->SubItems[COLUMN_POS_CLICK_NOISE_COUNT]->Text = gcnew String(af->GetClickNoiseCountString().c_str());
			item->SubItems[COLUMN_POS_REPAIR_CLICK_NOISE]->Text = gcnew String(af->GetDeClickNoiseString().c_str());
		}
	}
}

void FormMain::CallbackCommonFinish(TGenericCallbackParam^ param) {
	mainToolStripProgressBar->Value = 0;
	FixMenu();
}

void FormMain::LoadPrefFile() {
	//theGlobal->thePref->Load();
	//theGlobal->thePrefSaved = theGlobal->thePref->Clone();

	this->DesktopBounds = Prefs::data->windowBounds;

	List<int>^ widthList = Prefs::data->GetColumnWidth();
	for (int i = 0; i < widthList->Count; i++) {
		lvFileList->Columns[i]->Width = widthList[i];
	}

	std::wstring encoderName = TStringUtil::MarshalString(Prefs::data->encoder);
	for (int i = 0; i < (int)m_files->m_encoders.size(); i++) {
		if (m_files->m_encoders[i]->GetName() == encoderName && m_files->m_encoders[i]->IsNull() == false)
			m_files->m_selectedEncoder = m_files->m_encoders[i];
	}
	m_files->m_priority = Prefs::data->priority;
	expToolStripMenuItem->Visible = Prefs::data->exp;
	lvFileList->Font = gcnew ::Font(Prefs::data->fontFamily, Prefs::data->fontSize);
}

void FormMain::SavePrefFile() 
{
	if (WindowState == FormWindowState::Normal) {
		Prefs::data->windowBounds = this->DesktopBounds;
	}

	List<int>^ widthList = gcnew List<int>;
	for (int i = 0; i < lvFileList->Columns->Count; i++) {
		widthList->Add(lvFileList->Columns[i]->Width);
	}
	Prefs::data->SetColumnWidth(widthList);

	Prefs::data->encoder = gcnew String(m_files->m_selectedEncoder->GetName().c_str());

	Prefs::data->Save();
	//if (theGlobal->thePref->equals(theGlobal->thePrefSaved) == false) {
	//	theGlobal->thePref->Save();
	//}
}

void FormMain::FixMenu() {
	saveID3ToolStripMenuItem->Checked = Prefs::data->saveID3;
	selectACMCodecToolStripMenuItem->Checked = m_files->m_selectedEncoder == &m_files->m_acmEncoder;
	selectLameToolStripMenuItem->Checked = m_files->m_selectedEncoder == &m_files->m_lameEncoder;
	std::wstring s = _T("AudioFixLite [");
	s += m_files->m_selectedEncoder->GetName();
	wchar_t br[32];
	swprintf(br, _T(" %d]"), m_files->m_selectedEncoder->GetBitrate());
	s += std::wstring(br);
	String^ title = gcnew String(s.c_str());
	if (this->Text->Equals(title) == false)
		this->Text = gcnew String(s.c_str());

	for (int i = 0; i < m_priorities->Count; i++) {
		MenuItemValue^ iv = (MenuItemValue^)m_priorities->default[i];
		iv->m_item->Checked = (iv->m_value == m_files->m_priority);
	}

	if (m_files->IsWorking()) {
		startEncodeToolStripMenuItem->Enabled = false;
		stopEncodeToolStripMenuItem->Enabled = true;
		addWAVEFileToolStripMenuItem->Enabled = false;
		saveID3ToolStripMenuItem->Enabled = false;
		setupACMCodecToolStripMenuItem->Enabled = false;
		setupLameToolStripMenuItem->Enabled = false;
		saveFilesToToolStripMenuItem->Enabled = false;
		saveMergeFileToolStripMenuItem->Enabled = false;
		saveWaveFileToolStripMenuItem->Enabled = false;
		checkMaxToolStripMenuItem->Enabled = false;
		checkPhaseGapToolStripMenuItem->Enabled = false;
		checkClickNoiseToolStripMenuItem->Enabled = false;
	}
	else {
		startEncodeToolStripMenuItem->Enabled = true;
		stopEncodeToolStripMenuItem->Enabled = false;
		addWAVEFileToolStripMenuItem->Enabled = true;
		saveID3ToolStripMenuItem->Enabled = true;
		setupACMCodecToolStripMenuItem->Enabled = true;
		setupLameToolStripMenuItem->Enabled = true;
		saveFilesToToolStripMenuItem->Enabled = true;
		saveMergeFileToolStripMenuItem->Enabled = true;
		checkMaxToolStripMenuItem->Enabled = true;
		checkPhaseGapToolStripMenuItem->Enabled = true;
		checkClickNoiseToolStripMenuItem->Enabled = true;
		if (m_files->size() > 1) {
			saveMergeFileToolStripMenuItem->Enabled = true;
		}
		else {
			saveMergeFileToolStripMenuItem->Enabled = false;
		}
		if (m_files->size() > 0) {
			startEncodeToolStripMenuItem->Enabled = true;
			saveFilesToToolStripMenuItem->Enabled = true;
			checkMaxToolStripMenuItem->Enabled = true;
			checkPhaseGapToolStripMenuItem->Enabled = true;
			checkClickNoiseToolStripMenuItem->Enabled = true;
			gainToolStripMenuItem->Enabled = true;
			movePhaseToolStripMenuItem->Enabled = true;
			repairClickNoiseToolStripMenuItem->Enabled = true;
			repairDCOffsetToolStripMenuItem->Enabled = true;

			int selectedCount = 0;
			for (int i = 0; i < lvFileList->Items->Count; i++) {
				ListViewItem^ item = lvFileList->Items[i];
				if (item->Selected) {
					selectedCount++;
				}
			}
			saveWaveFileToolStripMenuItem->Enabled = selectedCount == 1;
		}
		else {
			startEncodeToolStripMenuItem->Enabled = false;
			saveFilesToToolStripMenuItem->Enabled = false;
			saveWaveFileToolStripMenuItem->Enabled = false;
			checkMaxToolStripMenuItem->Enabled = false;
			checkPhaseGapToolStripMenuItem->Enabled = false;
			checkClickNoiseToolStripMenuItem->Enabled = false;
			gainToolStripMenuItem->Enabled = false;
			movePhaseToolStripMenuItem->Enabled = false;
			repairClickNoiseToolStripMenuItem->Enabled = false;
			repairDCOffsetToolStripMenuItem->Enabled = false;
		}
	}
}

void FormMain::SetPriority(ToolStripMenuItem^ item) {
	for (int i = 0; i < m_priorities->Count; i++) {
		MenuItemValue^ iv = (MenuItemValue^)m_priorities->default[i];
		if (iv->m_item == item) {
			m_files->SetPriority(iv->m_value);
			break;
		}
	}
}

void FormMain::SaveFilesTo() {
	System::Windows::Forms::DialogResult result;
	if (Prefs::data->saveToDirPath->Length == 0)
		Prefs::data->saveToDirPath = Prefs::data->waveDirPath;
	saveWaveFilesToDialog->SelectedPath = Prefs::data->saveToDirPath;
	result = saveWaveFilesToDialog->ShowDialog();
	if (result != System::Windows::Forms::DialogResult::OK)
		return;
	Prefs::data->saveToDirPath = saveWaveFilesToDialog->SelectedPath;
	m_files->StartSaveFilesTo(TStringUtil::MarshalString(Prefs::data->saveToDirPath));
}

void FormMain::SaveMergeFile() {
	saveMergeFileDialog->Filter = gcnew String(_T("WAVE files|*.wav;*.wave|All files (*.*)|*.*"));
	saveMergeFileDialog->InitialDirectory = Prefs::data->waveDirPath;
	System::Windows::Forms::DialogResult result;
	result = saveMergeFileDialog->ShowDialog();
	if (result != System::Windows::Forms::DialogResult::OK)
		return;
	if (saveMergeFileDialog->FileName->Length == 0)
		return;
	m_files->StartSaveMergeFile(TStringUtil::MarshalString(saveMergeFileDialog->FileName));
}

std::wstring MakeRevFilePath(const std::wstring &path) {
	wchar_t drive[_MAX_DRIVE];
	wchar_t dir[_MAX_DIR];
	wchar_t fname[_MAX_FNAME];
	wchar_t ext[_MAX_EXT];
	_wsplitpath(path.c_str(), drive, dir, fname, ext);
	if (wcslen(fname) > _MAX_FNAME - 6)
		return path;
	wchar_t s[_MAX_PATH];
	for (int i = 1; i < 10000; i++) {
		wchar_t rfname[_MAX_FNAME];
		swprintf(rfname, L"%s_%d", fname, i);
		_wmakepath_s(s, _MAX_PATH, drive, dir, rfname, ext);
		struct _stat status;
		if (_wstat(s, &status) == -1)
			break;
	}
	return s;
}

void FormMain::SaveFileAs() {
	int count = lvFileList->Items->Count;
	if (count == 0)
		return;
	TAudioFile *af = NULL;
	for (int i = 0; i < count; i++) {
		ListViewItem^ item = lvFileList->Items[i];
		if (item->Selected) {
			if (af != NULL)
				return;
			af = m_files->at(i);
		}
	}
	if (af == NULL)
		return;
	m_saveFileAsOrgFilePath = gcnew String(af->m_waveFilePath.c_str());
	saveWaveFileDialog->Filter = gcnew String(_T("WAVE files|*.wav;*.wave|All files (*.*)|*.*"));
	//saveWaveFileDialog->InitialDirectory = Prefs::data->waveDirPath;
	saveWaveFileDialog->FileName = gcnew String(MakeRevFilePath(af->m_waveFilePath.c_str()).c_str());
	System::Windows::Forms::DialogResult result;
	result = saveWaveFileDialog->ShowDialog();
	if (result != System::Windows::Forms::DialogResult::OK)
		return;
	if (saveWaveFileDialog->FileName->Length == 0)
		return;
	m_files->StartSaveFileAs(af, TStringUtil::MarshalString(saveWaveFileDialog->FileName));
}

void FormMain::Gain() {
	int count = lvFileList->Items->Count;
	if (count == 0)
		return;
	int gainCmd = m_files->at(0)->m_gainCmd;
	double gain = m_files->at(0)->m_gain;
	for (int i = 0; i < count; i++) {
		ListViewItem^ item = lvFileList->Items[i];
		if (item->Selected) {
			gainCmd = m_files->at(i)->m_gainCmd;
			gain = m_files->at(i)->m_gain;
			break;
		}
	}

	System::Windows::Forms::DialogResult result;
	FormGainDlg^ dlg = gcnew FormGainDlg();
	dlg->SetCmd(gainCmd);
	dlg->SetGain(gain);
	dlg->SetLaud(Prefs::data->gainLaud);
	result = dlg->ShowDialog(this);
	if (result != System::Windows::Forms::DialogResult::OK)
		return;
	gainCmd = dlg->GetCmd();
	gain = dlg->GetGain();
	Prefs::data->gainLaud = dlg->GetLaud();
	for (int i = 0; i < count; i++) {
		ListViewItem^ item = lvFileList->Items[i];
		if (lvFileList->SelectedItems->Count == 0 || item->Selected) {
			m_files->at(i)->m_gainCmd = gainCmd;
			if (m_files->at(i)->m_gainCmd == GAIN_CMD_GAIN) {
				m_files->at(i)->m_gain = TAudioFile::RoundDecibel(gain);
			}
			else if (m_files->at(i)->m_gainCmd == GAIN_CMD_LAUD) {
				if (m_files->at(i)->m_maxLaudDecibel != 0.0) {
					double g = TAudioFile::RoundDecibel(Prefs::data->gainLaud - m_files->at(i)->m_maxLaudDecibel);
					if (g + m_files->at(i)->m_maxDecibel > -0.01) {
						g = TAudioFile::RoundDecibel(-0.01 - m_files->at(i)->m_maxDecibel);
					}
					m_files->at(i)->m_gain = g;
				}
			}
			lvFileList->Items[i]->SubItems[COLUMN_POS_GAIN]->Text = gcnew String(m_files->at(i)->GetGainString().c_str());
		}
	}
}

void FormMain::MovePhase() {
	int count = lvFileList->Items->Count;
	if (count == 0)
		return;
	int phaseCmd = m_files->at(0)->m_phaseCmd;
	int phaseMove = m_files->at(0)->m_phaseMove;
	for (int i = 0; i < count; i++) {
		ListViewItem^ item = lvFileList->Items[i];
		if (item->Selected) {
			phaseCmd = m_files->at(i)->m_phaseCmd;
			phaseMove = m_files->at(i)->m_phaseMove;
			break;
		}
	}

	System::Windows::Forms::DialogResult result;
	FormMovePhaseDlg^ dlg = gcnew FormMovePhaseDlg();
	dlg->SetCmd(phaseCmd);
	dlg->SetImmediate(phaseMove);
	result = dlg->ShowDialog(this);
	if (result != System::Windows::Forms::DialogResult::OK)
		return;
	phaseCmd = dlg->GetCmd();
	phaseMove = dlg->GetImmediate();
	for (int i = 0; i < count; i++) {
		ListViewItem^ item = lvFileList->Items[i];
		if (lvFileList->SelectedItems->Count == 0 || item->Selected) {
			m_files->at(i)->m_phaseCmd = phaseCmd;
			if (m_files->at(i)->m_phaseCmd == PHASE_CMD_AS_AVERAGE)
				m_files->at(i)->m_phaseMove = -m_files->at(i)->m_phaseGapAverage;
			else if (m_files->at(i)->m_phaseCmd == PHASE_CMD_MOVE)
				m_files->at(i)->m_phaseMove = phaseMove;
			lvFileList->Items[i]->SubItems[COLUMN_POS_MOVE_PHASE]->Text = gcnew String(m_files->at(i)->GetMovePhaseString().c_str());
		}
	}
}

void FormMain::Differentiate() {
/*
	int count = lvFileList->Items->Count;
	if (count == 0)
		return;
	bool sw = !m_files->at(0)->m_differentiate;
	for (int i = 0; i < count; i++) {
		m_files->at(i)->m_differentiate = sw;
	}
	yToolStripMenuItem->Checked = sw;
*/
}

void FormMain::CheckMaxDecibel() {
	m_files->StartCheckMaxDecibel();
}

void FormMain::CheckPhaseGap() {
	m_files->StartCheckPhaseGap();
}

void FormMain::CheckClickNoise() {
	System::Windows::Forms::DialogResult result;
	FormCheckClickNoiseDlg^ dlg = gcnew FormCheckClickNoiseDlg();
	dlg->SetThrAvrMag1(Prefs::data->clickNoiseThrAvrMag1);
	dlg->SetThrAvrMag2(Prefs::data->clickNoiseThrAvrMag2);
	dlg->SetThrFix(Prefs::data->clickNoiseThrFix);
	dlg->SetSaveLog(Prefs::data->saveLogOnCheckClickNoise);
	dlg->SetRepair(Prefs::data->repairOnCheckClickNoise);
	dlg->SetLeft(Prefs::data->clickNoiseLeft);
	dlg->SetRight(Prefs::data->clickNoiseRight);
	if (m_files->size() == 1) {
		dlg->SetStart(m_files->at(0)->m_noiseCheckStart);
		dlg->SetEnd(m_files->at(0)->m_noiseCheckEnd);
	}
	else {
		dlg->SetStart(0);
		dlg->SetEnd(-1);
		dlg->DisableStartEnd();
	}
	result = dlg->ShowDialog(this);
	if (result != System::Windows::Forms::DialogResult::OK)
		return;
	Prefs::data->clickNoiseThrAvrMag1 = dlg->GetThrAvrMag1();
	Prefs::data->clickNoiseThrAvrMag2 = dlg->GetThrAvrMag2();
	Prefs::data->clickNoiseThrFix = dlg->GetThrFix();
	Prefs::data->saveLogOnCheckClickNoise = dlg->GetSaveLog();
	Prefs::data->repairOnCheckClickNoise = dlg->GetRepair();
	Prefs::data->clickNoiseLeft = dlg->GetLeft();
	Prefs::data->clickNoiseRight = dlg->GetRight();
	if (m_files->size() == 1) {
		m_files->at(0)->m_noiseCheckStart = dlg->GetStart();
		m_files->at(0)->m_noiseCheckEnd = dlg->GetEnd();
	}
	else {
		for (int i = 0; i < (int)m_files->size(); i++) {
			m_files->at(i)->m_noiseCheckStart = 0;
			m_files->at(i)->m_noiseCheckEnd = -1;
		}
	}

	m_files->StartCheckClickNoise();
}

void FormMain::RepairClickNoise() {
	int count = lvFileList->Items->Count;
	if (count == 0)
		return;
	int cmd = m_files->at(0)->m_deClickNoiseCmd;
	for (int i = 0; i < count; i++) {
		ListViewItem^ item = lvFileList->Items[i];
		if (item->Selected) {
			cmd = m_files->at(i)->m_deClickNoiseCmd;
			break;
		}
	}

	System::Windows::Forms::DialogResult result;
	FormRepairClickNoiseDlg^ dlg = gcnew FormRepairClickNoiseDlg();
	dlg->SetCmd(cmd);
	result = dlg->ShowDialog(this);
	if (result != System::Windows::Forms::DialogResult::OK)
		return;
	cmd = dlg->GetCmd();

	for (int i = 0; i < count; i++) {
		ListViewItem^ item = lvFileList->Items[i];
		if (lvFileList->SelectedItems->Count == 0 || item->Selected) {
			m_files->at(i)->m_deClickNoiseCmd = cmd;
			lvFileList->Items[i]->SubItems[COLUMN_POS_REPAIR_CLICK_NOISE]->Text = gcnew String(m_files->at(i)->GetDeClickNoiseString().c_str());
		}
	}
}

void FormMain::DCOffset() {
	int count = lvFileList->Items->Count;
	if (count == 0)
		return;
	double left = 0.0;
	double right = 0.0;
	bool leftDown = true;
	bool rightDown = true;
	for (int i = 0; i < count; i++) {
		ListViewItem^ item = lvFileList->Items[i];
		if (item->Selected || i == 0) {
			left = m_files->at(i)->m_dcOffsetLeft;
			leftDown = m_files->at(i)->m_dcOffsetLeftDown;
			right = m_files->at(i)->m_dcOffsetRight;
			rightDown = m_files->at(i)->m_dcOffsetRightDown;
			if (left == 0.0 && right == 0.0) {
				left = m_files->at(i)->m_averageLeft;
				leftDown = m_files->at(i)->m_averageLeftPlus;
				right = m_files->at(i)->m_averageRight;
				rightDown = m_files->at(i)->m_averageRightPlus;
			}
			if (item->Selected)
				break;
		}
	}

	System::Windows::Forms::DialogResult result;
	FormDCOffsetDlg^ dlg = gcnew FormDCOffsetDlg();
	dlg->SetLeft(left);
	dlg->SetLeftDown(leftDown);
	dlg->SetRight(right);
	dlg->SetRightDown(rightDown);
	result = dlg->ShowDialog(this);
	if (result != System::Windows::Forms::DialogResult::OK)
		return;
	left = dlg->GetLeft();
	leftDown = dlg->GetLeftDown();
	right = dlg->GetRight();
	rightDown = dlg->GetRightDown();
	for (int i = 0; i < count; i++) {
		ListViewItem^ item = lvFileList->Items[i];
		if (lvFileList->SelectedItems->Count == 0 || item->Selected) {
			m_files->at(i)->m_dcOffsetLeft = TAudioFile::RoundDecibel(left);
			m_files->at(i)->m_dcOffsetLeftDown = leftDown;
			m_files->at(i)->m_dcOffsetRight = TAudioFile::RoundDecibel(right);
			m_files->at(i)->m_dcOffsetRightDown = rightDown;
			lvFileList->Items[i]->SubItems[COLUMN_POS_MOVE_DC_OFFSET]->Text = gcnew String(m_files->at(i)->GetMoveDCOffsetString().c_str());
		}
	}
}

System::Void FormMain::fontToolStripMenuItem_Click(System::Object^  sender, System::EventArgs^  e) {
	fontDialog->Font = lvFileList->Font;
	fontDialog->ShowEffects = false;
	fontDialog->ShowColor = false;
	if (fontDialog->ShowDialog(this) != ::DialogResult::OK) {
		return;
	}
	Prefs::data->fontFamily = fontDialog->Font->FontFamily->Name;
	Prefs::data->fontSize = fontDialog->Font->Size;
	lvFileList->Font = gcnew ::Font(Prefs::data->fontFamily, Prefs::data->fontSize);

	FixMenu();
}

System::Void FormMain::preferencesToolStripMenuItem_Click(System::Object^  sender, System::EventArgs^  e) {
	FormPreferencesDlg^ dlg = gcnew FormPreferencesDlg();
	dlg->SetSaveID3(Prefs::data->saveID3);
	dlg->SetApplyFontToDialog(Prefs::data->applyFontToDialog);
	if (dlg->ShowDialog(this) != ::DialogResult::OK) {
		return;
	}
	Prefs::data->saveID3 = dlg->GetSaveID3();
	Prefs::data->applyFontToDialog = dlg->GetApplyFontToDialog();
	FixMenu();
}

}	// namespace

