#include "stdafx.h"
#include "TGlobal.h"
#include "TAudioFile.h"

TAudioFile::TAudioFile() {
	m_status = 0;
	m_gainCmd = GAIN_CMD_GAIN;
	m_gain = 0.0;
	m_maxSample = 0;
	m_maxDecibel = 0.0;
	m_maxSamplePos = 0;
	m_maxLaudSample = 0;
	m_maxLaudDecibel = 0.0;
	m_phaseGapAverage = 0;
	m_phaseGapCheckStatus = GAP_CHECK_STATUS_NON;
	m_phaseCmd = PHASE_CMD_NON;
	m_phaseMove = 0;
	m_frameCount = 0;
	m_deClickNoiseCmd = DECLICKNOISE_CMD_NON;
	m_noiseCheckStart = 0;
	m_noiseCheckEnd = -1;
	m_dcOffsetLeft = 0.0;
	m_dcOffsetLeftDown = true;
	m_dcOffsetRight = 0.0;
	m_dcOffsetRightDown = true;
	m_averageLeft = 0.0;
	m_averageLeftPlus = true;
	m_averageRight = 0.0;
	m_averageRightPlus = true;
}

TAudioFile::~TAudioFile() {
}

bool TAudioFile::Setup(const std::wstring &waveFilePath) {
	if (m_format.CheckAudioFileFormat(waveFilePath.c_str()) == false)
		return false;
	m_sampleBytes = BitsToBytes(m_format.m_waveFormat.wBitsPerSample);
	m_frameCount = (m_format.m_dataBytes / m_sampleBytes) / m_format.m_waveFormat.nChannels;
	m_waveFilePath = waveFilePath;
	m_mp3FilePath = ChangeExtension(waveFilePath, _T(".mp3"));
	m_ccnFilePath = ChangeExtension(waveFilePath, _T(".ccn"));
	m_ccuFilePath = ChangeExtension(waveFilePath, _T(".ccu"));
	m_status = 0;
	MakeID3Data();
	return true;
}

bool TAudioFile::CheckExtension(const std::wstring &path, const std::wstring &ext) {
	wchar_t buffer[_MAX_EXT];
	_wsplitpath(path.c_str(), NULL, NULL, NULL, buffer);
	return _wcsicmp(buffer, ext.c_str()) == 0;
}

std::wstring TAudioFile::GetStatusString() {
	if (m_status == 0) {
		return _T("-");
	}
	else if (m_status == 100) {
		return _T("ok");
	}
	else if (m_status <= -100) {
		return _T("error");
	}
	else {
		wchar_t s[16];
		swprintf(s, _T("%d"), m_status);
		return s;
	}
}

std::wstring TAudioFile::GetFileName(const std::wstring &path) {
	wchar_t fname[_MAX_FNAME];
	wchar_t ext[_MAX_EXT];
	_wsplitpath(path.c_str(), NULL, NULL, fname, ext);
	wchar_t s[_MAX_PATH];
	_wmakepath_s(s, _MAX_PATH, _T(""), _T(""), fname, ext);
	return s;
}

std::wstring TAudioFile::GetWaveFileName() {
	return GetFileName(m_waveFilePath);
}

std::wstring TAudioFile::GetMp3FileName() {
	return GetFileName(m_mp3FilePath);
}

std::wstring TAudioFile::GetRoundDecibelString(double db) {
	wchar_t s[32];
	swprintf(s, _T("%.2f"), db);
	return s;
}

double TAudioFile::RoundDecibel(double db) {
	std::wstring s = GetRoundDecibelString(db);
	double result = db;
	try {
		result = double::Parse(gcnew String(s.c_str()));
	}
	catch (Exception^) {
	}
	return result;
}

std::wstring TAudioFile::GetMaxDecibelString() {
	if (m_maxSample == 0 && m_maxDecibel == 0.0)
		return _T("-");
	return GetRoundDecibelString(m_maxDecibel);
//	wchar_t s[32];
//	swprintf(s, _T("%.2f"), m_maxDecibel);
//	return s;
}

std::wstring TAudioFile::GetGainString() {
	return GetRoundDecibelString(m_gain);
//	wchar_t s[32];
//	swprintf(s, _T("%.2f"), m_gain);
//	return s;
}

std::wstring TAudioFile::GetPhaseGapAverageString() {
	if (m_phaseGapCheckStatus == GAP_CHECK_STATUS_NON)
		return _T("-");
	if (m_phaseGapCheckStatus == GAP_CHECK_STATUS_NG)
		return _T("x");
	wchar_t s[32];
	swprintf(s, _T("%d (%d%%)"), m_phaseGapAverage, m_phaseGapPercent);
	return s;
}

std::wstring TAudioFile::GetMovePhaseString() {
	if (m_phaseCmd == PHASE_CMD_NON)
		return _T("-");
	wchar_t s[32];
	swprintf(s, _T("%d"), m_phaseMove);
	return s;
}

std::wstring TAudioFile::GetMaxPosString() {
	if (m_maxSample == 0 && m_maxDecibel == 0.0)
		return _T("-");
	wchar_t s[32];
	swprintf(s, _T("%d"), m_maxSamplePos);
	return s;
}

std::wstring TAudioFile::GetMaxLaudDecibelString() {
	if (m_maxLaudSample == 0 && m_maxLaudDecibel == 0.0)
		return _T("-");
	return GetRoundDecibelString(m_maxLaudDecibel);
//	wchar_t s[32];
//	swprintf(s, _T("%.2f"), m_maxLaudDecibel);
//	return s;
}

std::wstring TAudioFile::GetClickNoiseCountString() {
	if (m_noiseFramePos.size() == 0 || m_frameCount == 0)
		return _T("-");
	int c = 0;
	for (int i = 0; i < (int)m_noiseFramePos.size(); i++)
		c += m_noiseFramePos[i].size();
	double percent = (double)(100 * m_noiseFramePos.GetFramesSum()) / (double)(m_frameCount * m_format.m_waveFormat.nChannels);
	wchar_t s[32];
	swprintf(s, _T("%d (%.3f%%)"), c, percent);
	return s;
}

std::wstring TAudioFile::GetDeClickNoiseString() {
	if (m_deClickNoiseCmd == DECLICKNOISE_CMD_DECLICK)
		return _T("R");
	else if (m_deClickNoiseCmd == DECLICKNOISE_CMD_DY1) {
		return _T("y'");
	}
	else if (m_deClickNoiseCmd == DECLICKNOISE_CMD_DY2) {
		return _T("y''");
	}
	else if (m_deClickNoiseCmd == DECLICKNOISE_CMD_NOISE) {
		return _T("N");
	}
	return _T("-");
}

std::wstring TAudioFile::GetAverageString() {
	if (m_averageLeft == 0.0 && m_averageRight == 0.0) {
		return _T("-");
	}
	else {
		wchar_t left[32];
		wchar_t right[32];
		if 	(m_averageLeft < -90.0) {
			swprintf(left, _T("%s %s"), _T("- dB"), m_averageLeftPlus ? _T("U") : _T("D"));
		}
		else {
			swprintf(left, _T("%.3f dB %s"), m_averageLeft, m_averageLeftPlus ? _T("U") : _T("D"));
		}
		if 	(m_averageRight < -90.0) {
			swprintf(right, _T("%s %s"), _T("- dB"), m_averageRightPlus ? _T("U") : _T("D"));
		}
		else {
			swprintf(right, _T("%.3f dB %s"), m_averageRight, m_averageRightPlus ? _T("U") : _T("D"));
		}
		wchar_t s[64];
		swprintf(s, _T("L %s, R %s"), left, right);
		return s;
	}
}

std::wstring TAudioFile::GetMoveDCOffsetString() {
	if (m_dcOffsetLeft == 0.0 && m_dcOffsetRight == 0.0) {
		return _T("-");
	}
	else {
		wchar_t left[32];
		wchar_t right[32];
		swprintf(left, _T("%.3f dB %s"), m_dcOffsetLeft, m_dcOffsetLeftDown ? _T("D") : _T("U"));
		swprintf(left, _T("%.3f dB %s"), m_dcOffsetRight, m_dcOffsetRightDown ? _T("D") : _T("U"));
		wchar_t s[64];
		swprintf(s, _T("L %s, R %s"), left, right);
		return s;
	}
}

std::wstring TAudioFile::GetHzString() {
	wchar_t s[32];
	swprintf(s, _T("%d"), m_format.m_waveFormat.nSamplesPerSec);
	return s;
}

std::wstring TAudioFile::GetBitsString() {
	wchar_t s[32];
	swprintf(s, _T("%d"), m_format.m_waveFormat.wBitsPerSample);
	return s;
}

std::wstring TAudioFile::GetChString() {
	wchar_t s[32];
	swprintf(s, _T("%d"), m_format.m_waveFormat.nChannels);
	return s;
}

std::wstring TAudioFile::GetTimeString() {
	long sec = m_format.CalcTimeLong();
	wchar_t s[32];
	swprintf(s, _T("%d:%02d"), sec / 60, sec % 60);
	return s;
}

std::wstring TAudioFile::ChangeExtension(const std::wstring &path, const std::wstring &newExt) {
	wchar_t drive[_MAX_DRIVE];
	wchar_t dir[_MAX_DIR];
	wchar_t fname[_MAX_FNAME];
	wchar_t ext[_MAX_EXT];
	_wsplitpath(path.c_str(), drive, dir, fname, ext);
	wchar_t s[_MAX_PATH];
	_wmakepath_s(s, _MAX_PATH, drive, dir, fname, newExt.c_str());
	return s;
}

std::wstring TAudioFile::ChangeDirectory(const std::wstring &path, const std::wstring &newDir) {
	wchar_t drive[_MAX_DRIVE];
	wchar_t dir[_MAX_DIR];
	wchar_t fname[_MAX_FNAME];
	wchar_t ext[_MAX_EXT];
	_wsplitpath(path.c_str(), drive, dir, fname, ext);
	wchar_t s[_MAX_PATH];
	_wmakepath_s(s, _MAX_PATH, _T(""), newDir.c_str(), fname, ext);
	return s;
}

int TAudioFile::MakeV2Frame(BYTE *frame, char *frameID, int sz) {
	BYTE *p = frame;
	int frameIDSize = 3;
	memcpy(p, (char *)frameID, frameIDSize);
	p += frameIDSize;
	int frameSizeSize = 3;
	for (int i = 0; i < frameSizeSize; i++) {
		*p = ((sz >> (8 * (frameSizeSize - 1 - i))) & 0xFF);
		p++;
	}
	return p - frame;
}

int TAudioFile::MakeV2TextFrame(BYTE *frame, char *frameID, const std::wstring &data) {
	if (data.empty())
		return 0;
	int sz = data.size() * sizeof(wchar_t) + 1 /* encoding */ + sizeof(wchar_t) /* BOM */;
	BYTE *p = frame;
	p += MakeV2Frame(p, frameID, sz);
	*p = CODE_UTF16;
	p++;
	*p = 0xFF;
	p++;
	*p = 0xFE;
	p++;
	memcpy(p, data.c_str(), data.size() * sizeof(wchar_t));
	p += data.size() * sizeof(wchar_t);
	return p - frame;
}

void TAudioFile::SetV2TagSize(ID3_V2_HEAD *head, int sz) {
	head->m_size[3] = (sz & 0x7F);
	head->m_size[2] = ((sz >> 7) & 0x7F);
	head->m_size[1] = ((sz >> 7 * 2) & 0x7F);
	head->m_size[0] = ((sz >> 7 * 3) & 0x7F);
}

std::wstring lstrip(std::wstring& s) {
	std::wstring r = _T("");
	for (int i = 0; i < (int)s.size(); i++) {
		if (s[i] == 0x20 && r.size() == 0) {
		}
		else {
			r += s[i];
		}
	}
	return r;
}

void TAudioFile::MakeID3Data() {
	wchar_t drive[_MAX_DRIVE];
	wchar_t dir[_MAX_DIR];
	wchar_t fname[_MAX_FNAME];
	wchar_t ext[_MAX_EXT];
	_wsplitpath(m_mp3FilePath.c_str(), drive, dir, fname, ext);
	if (fname[0] == _T('[')) {
		std::stack<std::wstring> k;
		wchar_t *t = wcstok(fname, _T("[]"));
		for (; t != NULL;) {
			k.push(t);
			t = wcstok(NULL, _T("\\"));
		}
		if (k.size() > 0) {
			m_title = lstrip(k.top());
			k.pop();
		}
		if (k.size() > 0) {
			m_artist = k.top();
			k.pop();
		}
	}
	else {
		m_title = fname;
		std::stack<std::wstring> k;
		wchar_t *t = wcstok(dir, _T("\\"));
		for (; t != NULL;) {
			k.push(t);
			t = wcstok(NULL, _T("\\"));
		}
		if (k.size() > 0) {
			m_album = k.top();
			k.pop();
		}
		if (k.size() > 0) {
			m_artist = k.top();
			k.pop();
		}
	}
}

void TAudioFile::SaveID3Tag(TFileWriter *fw) {
	MakeID3Data();
	/* GcȃTCYvZ */
	int sz = 200 +
	         m_title.size() * sizeof(wchar_t) +
	         m_artist.size() * sizeof(wchar_t) +
	         m_album.size() * sizeof(wchar_t);
	BYTE *data = new BYTE [sz];
	memset(data, 0, sz);
	BYTE *p = data;

	ID3_V2_HEAD *head = (ID3_V2_HEAD *)p;
	memcpy(head->m_tag, "ID3", sizeof(head->m_tag));
	head->m_major = TAG_VERSION_2_2;
	head->m_minor = 0;
	p += sizeof(ID3_V2_HEAD);

	p += MakeV2TextFrame(p, "TT2", m_title);
	p += MakeV2TextFrame(p, "TP1", m_artist);
	p += MakeV2TextFrame(p, "TAL", m_album);
	p += MakeV2TextFrame(p, "TCO", _T("(13)"));

	SetV2TagSize(head, p - data - sizeof(ID3_V2_HEAD));

	int length = p - data;
	fw->Write(data, length);
	delete [] data;
}

long TAudioFile::GetPrefUnitCount() {
	return Prefs::data->unitCount;
}

long TAudioFile::GetPrefEdgeCount() {
	return Prefs::data->edgeCount;
}

double TAudioFile::GetPrefCheckLevel() {
	return Prefs::data->checkLevel;
}

long TAudioFile::GetPrefGapHint() {
	return Prefs::data->gapHint;
}

bool TAudioFile::GetPrefEnableOne() {
	return Prefs::data->enableOne;
}

bool TAudioFile::GetPrefSaveLogOnCheckClickNoise() {
	return Prefs::data->saveLogOnCheckClickNoise;
}

bool TAudioFile::GetPrefRepairOnCheckClickNoise() {
	return Prefs::data->repairOnCheckClickNoise;
}

int TAudioFile::GetPrefClickNoiseAvrCount() {
	return Prefs::data->clickNoiseAvrCount;
}

#pragma managed(push, off)

TProcessContext::TProcessContext(TAudioFileProcess *context, size_t bytesSum) {
	m_context = context;
	m_bytesSum = bytesSum;
	m_prevStatus = 0;
	m_bytesSumRead = 0;
}

bool TProcessContext::IsContinue() {
	return m_context->m_continue;
}

void TProcessContext::Progress(size_t addBytes) {
	m_bytesSumRead += addBytes;
	int status = (int)(100.0 * ((double)m_bytesSumRead / (double)m_bytesSum));
	if (m_prevStatus != status) {
		m_context->CallCommonProgress(status);
		m_prevStatus = status;
	}
}

class TMaxDecibelChecker {
public:
	long m_maxSample;
	long m_maxSamplePos;
	TMaxDecibelChecker();
	void Check(long sample, long framePos);
};

TMaxDecibelChecker::TMaxDecibelChecker() {
	m_maxSample = 0;
	m_maxSamplePos = 0;
}

void TMaxDecibelChecker::Check(long sample, long framePos) {
	if (abs(sample) > abs(m_maxSample)) {
		m_maxSample = sample;
		m_maxSamplePos = framePos;
	}
}

class TMaxLaudChecker {
public:
	long m_laudPos;
	long long m_maxLaudSum;
	long long m_laudSum;
	long m_laudCacheSamples;
	bool m_bCheckLaud;
	long *m_laudCacheBuffer;
	TMaxLaudChecker(long laudCacheSamples);
	~TMaxLaudChecker();
	void Check(long sample);
	long GetMaxLaudSample();
};

TMaxLaudChecker::TMaxLaudChecker(long laudCacheSamples) {
	m_laudPos = 0;
	m_maxLaudSum = 0;
	m_laudSum = 0;
	m_laudCacheSamples = laudCacheSamples;
	m_bCheckLaud = false;
	m_laudCacheBuffer = new long[m_laudCacheSamples];
}

TMaxLaudChecker::~TMaxLaudChecker() {
	delete [] m_laudCacheBuffer;
}

void TMaxLaudChecker::Check(long sample) {
	long absSample = abs(sample);
	m_laudSum += absSample;
	if (m_bCheckLaud) {
		m_laudSum -= m_laudCacheBuffer[m_laudPos];
		if (m_maxLaudSum < m_laudSum) {
			m_maxLaudSum = m_laudSum;
		}
	}
	m_laudCacheBuffer[m_laudPos] = absSample;
	m_laudPos++;
	if (m_laudPos == m_laudCacheSamples) {
		m_bCheckLaud = true;
		m_laudPos = 0;
	}
}

long TMaxLaudChecker::GetMaxLaudSample() {
	return (long)(m_maxLaudSum / m_laudCacheSamples);
}

class TAverageChecker {
public:
	long long m_sumL;
	long long m_countL;
	long long m_sumR;
	long long m_countR;
	TAverageChecker() {
		m_sumL = 0;
		m_countL = 0;
		m_sumR = 0;
		m_countR = 0;
	}
	void Check(int ch, long sample) {
		if (ch == 0) {
			m_sumL += sample;
			m_countL++;
		}
		else if (ch == 1) {
			m_sumR += sample;
			m_countR++;
		}
	}
};

bool TAudioFile::CheckMaxDecibel(TProcessContext *context) {
	TWaveFileReader fIn;
	fIn.Open(m_waveFilePath, &m_format);

	TMaxDecibelChecker mdc;
	TMaxLaudChecker mlc(m_format.m_waveFormat.nSamplesPerSec * 5);
	TAverageChecker avc;
	bool good = true;

	for (long framePos = 0; framePos < m_frameCount; framePos++) {
		if (context->IsContinue() == false) {
			break;
		}
		for (int ch = 0; ch < m_format.m_waveFormat.nChannels; ch++) {
			long sample = fIn.GetSample(ch, framePos);
			long absSample = abs(sample);
			// őlB
			mdc.Check(absSample, framePos);
			// B
			mlc.Check(absSample);
			// ϒl
			avc.Check(ch, sample);
		}
		// progress
		context->Progress(1);
	}
	if (context->IsContinue() == false)
		return false;
	if (good) {
		m_maxSample = mdc.m_maxSample;
		m_maxSamplePos = mdc.m_maxSamplePos;
		TGainFilter gainFilter(m_format.m_waveFormat.wBitsPerSample, m_gain);
		m_maxDecibel = gainFilter.SampleToDecibel(m_maxSample);
		m_maxLaudSample = mlc.GetMaxLaudSample();
		m_maxLaudDecibel = gainFilter.SampleToDecibel(m_maxLaudSample);
		m_averageLeft = gainFilter.SampleToDecibel(avc.m_sumL / avc.m_countL);
		m_averageLeftPlus = avc.m_sumL > 0;
		m_averageRight = gainFilter.SampleToDecibel(avc.m_sumR / avc.m_countR);
		m_averageRightPlus = avc.m_sumR > 0;
	}

	return good;
}

bool TAudioFile::CheckPhaseGap(TProcessContext *context) {
	long unitCount = GetPrefUnitCount();
	long edgeCount = GetPrefEdgeCount();
	double checkLevel = GetPrefCheckLevel();
	long gapHint = GetPrefGapHint();
	bool enableOne = GetPrefEnableOne();
	m_phaseGapList.clear();
	if (m_format.m_waveFormat.nChannels != 2)	// Stereo ̂
		return false;
	long checkPoints = (m_frameCount / unitCount) - 2;
	if (checkPoints < 1)
		return false;
	
	TWaveFileReader fIn(m_waveFilePath, &m_format);

	TLongBuffer buff0Instance(unitCount);
	TLongBuffer buff1Instance(unitCount + (edgeCount * 2));
	long *buff0 = buff0Instance.m_buffer;
	long *buff1 = buff1Instance.m_buffer;

	TLongBuffer gapSumInstance(edgeCount * 2 + 1);	// _0̂ +1B
	long *gapSum = gapSumInstance.m_buffer;

	long checkp0 = unitCount;
	long checkp1 = unitCount - edgeCount + gapHint;

	long gapTotal = gapHint;

	TMaxDecibelChecker mdc;
	TMaxLaudChecker mlc(m_format.m_waveFormat.nSamplesPerSec * 5);
//	TMaxLaudChecker mlc(m_format.m_waveFormat.nChannels, m_format.m_waveFormat.nSamplesPerSec * 5);
	bool good = true;
	long loop = 0;
	long rest = m_frameCount;
	for (;
		loop < checkPoints && checkp1 + unitCount + (edgeCount * 2) <= m_frameCount && good;
		loop++) {
		if (context->IsContinue() == false) {
			return false;
		}

		for (long p = 0; p < unitCount; p++) {
//			buff0[p] = fIn.GetSample(0, checkp0 + p);
			long sample0 = fIn.GetSample(0, checkp0 + p);
			long sample1 = fIn.GetSample(1, checkp0 + p);
			buff0[p] = sample0;
			mdc.Check(sample0, checkp0 + p);
			mdc.Check(sample1, checkp0 + p);
			mlc.Check(sample0);
			mlc.Check(sample1);
//			mlc.Check(0, sample0, checkp0 + p);
//			mlc.Check(1, sample1, checkp0 + p);
		}
		for (long p = 0; p < unitCount + (edgeCount * 2); p++) {
			buff1[p] = fIn.GetSample(1, checkp1 + p);
		}

		context->Progress(unitCount);
		rest -= unitCount;

		if (CheckPhaseGapLevel(buff0, unitCount, checkLevel) == false) {
			checkp0 += unitCount;
			checkp1 += unitCount;
			continue;
		}

		double reference = CalcPhaseDiff(buff0, &buff1[edgeCount], unitCount);
		double next = CalcPhaseDiff(buff0, &buff1[edgeCount + 1], unitCount);
		double prev = CalcPhaseDiff(buff0, &buff1[edgeCount - 1], unitCount);

		long gap = 0;
		if (next < reference && next < prev) {
			double k = next;
			for (int i = 2; i <= edgeCount; i++) {
				double t = CalcPhaseDiff(buff0, &buff1[edgeCount + i], unitCount);
				if (t > k) {
					gap = i - 1;
					break;
				}
				k = t;
			}
			if (gap == 0)
				gap = edgeCount;
		}
		else if (prev < reference && prev < next) {
			double k = prev;
			for (int i = -2; i >= -edgeCount; i--) {
				double t = CalcPhaseDiff(buff0, &buff1[edgeCount + i], unitCount);
				if (t > k) {
					gap = i + 1;
					break;
				}
				k = t;
			}
			if (gap == 0)
				gap = -edgeCount;
		}
		if (enableOne == FALSE && (gap == 1 || gap == -1))		// ǂ -1  1 JԂX
			gap = 0;

		//
		if (abs(gapTotal + gap) > edgeCount) {
			checkp0 += unitCount;
			checkp1 += unitCount;
			continue;
		}
		//
		gapTotal += gap;
		if (gap != 0 || m_phaseGapList.size() == 0)
			m_phaseGapList.push_back(TPhaseGap(checkp0, gapTotal));
		checkp0 += unitCount;
		checkp1 += (unitCount + gap);
		if (gapTotal > edgeCount || -gapTotal > edgeCount) {
			good = false;
			//theMain->m_errorMessage = "Phase gap too large";
		}
		else {
			gapSum[edgeCount + gapTotal]++;
		}
	}

	//context->Progress(rest);

	if (good) {
		for (long p = loop * unitCount; p < m_frameCount; p++) {
			long sample0 = fIn.GetSample(0, p);
			long sample1 = fIn.GetSample(1, p);
			mdc.Check(sample0, p);
			mdc.Check(sample1, p);
			mlc.Check(sample0);
			mlc.Check(sample1);
//			mlc.Check(0, sample0, p);
//			mlc.Check(1, sample1, p);
			context->Progress(1);
		}

		int v = 0;
		for (int i = 0; i < edgeCount * 2; i++) {
			if (gapSum[i] > v) {
				m_phaseGapAverage = i - edgeCount;
				v = gapSum[i];
			}
		}
		m_phaseMove = -m_phaseGapAverage;
		m_phaseGapPercent = v * 100 / checkPoints;
		//
		m_maxSample = mdc.m_maxSample;
		m_maxSamplePos = mdc.m_maxSamplePos;
		TGainFilter gainFilter(m_format.m_waveFormat.wBitsPerSample, m_gain);
		m_maxDecibel = gainFilter.SampleToDecibel(m_maxSample);
		m_maxLaudSample = mlc.GetMaxLaudSample();
		m_maxLaudDecibel = gainFilter.SampleToDecibel(m_maxLaudSample);
		m_phaseGapCheckStatus = GAP_CHECK_STATUS_OK;
	}
	else { 
		m_phaseGapList.clear();
		if (context->IsContinue()) {
			m_phaseGapCheckStatus = GAP_CHECK_STATUS_NG;
		}
	}

	return good;
}

bool TAudioFile::CheckPhaseGapLevel(long *buff, long count, double checkLevel) {
	TGainFilter gainFilter(m_format.m_waveFormat.wBitsPerSample);
	for (int i = 0; i < count; i++) {
		if (gainFilter.SampleToDecibel(buff[i]) >= checkLevel)
			return true;
	}	
	return false;
}

double TAudioFile::CalcPhaseDiff(long *buff0, long *buff1, long count) {
	double result = 0.0;
	for (int i = 0; i < count; i++) {
		long diff = buff0[i] - buff1[i];
		if (diff < 0)
			diff = -diff;
		result += diff;
	}
	return result;
}

bool TAudioFile::CheckClickNoise(TProcessContext *context) {
	TWaveFileReader wfr;
	wfr.Open(m_waveFilePath, &m_format);

	std::vector<TClickNoiseChecker *> checker;
	m_noiseFramePos.Setup(m_format.m_waveFormat.nChannels);
	for (int i = 0; i < m_format.m_waveFormat.nChannels; i++) {
		checker.push_back(new TClickNoiseChecker(&wfr, i, m_frameCount, m_noiseCheckStart, m_noiseCheckEnd));
	}

	bool good = true;
	long st = m_noiseCheckStart - GetPrefClickNoiseAvrCount() * 2;
	if (st < 0)
		st = 0;
	long en = m_frameCount;
	if (m_noiseCheckEnd != -1) {
		en = m_noiseCheckEnd + GetPrefClickNoiseAvrCount() * 2;
		if (en > m_frameCount)
			en = m_frameCount;
	}
	for (long framePos = st ; framePos < en; framePos++) {
		if (context->IsContinue() == false) {
			break;
		}
		for (int ch = 0; ch < m_format.m_waveFormat.nChannels; ch++) {
			long sample = wfr.GetSample(ch, framePos);
			checker[ch]->CheckNoise(framePos, sample, &m_noiseFramePos/*[ch]*/);
		}
		// progress
		context->Progress(1);
	}
	if (context->IsContinue() == false)
		good = false;
	if (good) {
		if (GetPrefSaveLogOnCheckClickNoise()) {
			m_noiseFramePos.Save(m_ccnFilePath);
		}
		if (GetPrefRepairOnCheckClickNoise()) {
			m_deClickNoiseCmd = DECLICKNOISE_CMD_DECLICK;
		}
	}
	else {
		m_noiseFramePos.clear();
	}

	for (int i = 0; i < (int)checker.size(); i++) {
		delete checker[i];
	}
	checker.clear();

	return good;
}

#pragma managed(pop)

TAudioFileProcess::TAudioFileProcess() {
	m_thread = NULL;
	m_threadPrev = NULL;
	EncodeProgress = NULL;
	EncodeFinishOne = NULL;
	CommonProgress = NULL;
	CommonStartOne = NULL;
	CommonFinishOne = NULL;
	CommonFinish = NULL;
	m_acmEncoder.Init(this);
	m_lameEncoder.Init(this);
	m_encoders.push_back(&m_acmEncoder);
	m_encoders.push_back(&m_lameEncoder);
	m_selectedEncoder = &m_acmEncoder;
//	m_selectedEncoder = &m_lameEncoder;
	m_priority = THREAD_PRIORITY_LOWEST;
	m_threadCommand = 0;
}

TAudioFileProcess::~TAudioFileProcess() {
	if (m_threadPrev != NULL) {
		CloseHandle(m_threadPrev);
		m_threadPrev = NULL;
	}
/*
	Join();
	if (m_thread != NULL) {
		CloseHandle(m_thread);
		m_thread = NULL;
	}
*/
	if (IsWorking() == false) {
		RemoveDeleteAll();
	}
}

void TAudioFileProcess::RemoveDeleteAll() {
	for (size_t i = 0; i < size(); i++) {
		delete at(i);
	}
	clear();
}

void TAudioFileProcess::ClearAllStatus() {
	for (size_t i = 0; i < size(); i++) {
		at(i)->m_status = 0;
	}
}

unsigned CALLBACK TAudioFileProcess::ThreadMain(void *param) {
	TAudioFileProcess *context = (TAudioFileProcess *)param;
	bool result = false;
	if (context->m_threadCommand == THR_CMD_ENCODE) {
		result = context->Encode();
	}
	else if (context->m_threadCommand == THR_CMD_SAVE_FILES_TO) {
		result = context->SaveFilesTo(context->m_saveFilesToDirPath);
	}
	else if (context->m_threadCommand == THR_CMD_SAVE_MERGE_FILE) {
		result = context->SaveMergeFile(context->m_saveMergeFilePath);
	}
	else if (context->m_threadCommand == THR_CMD_CHECK_MAX_DECIBEL) {
		result = context->CheckMaxDecibel();
	}
	else if (context->m_threadCommand == THR_CMD_CHECK_PHASE_GAP) {
		result = context->CheckPhaseGap();
	}
	else if (context->m_threadCommand == THR_CMD_CHECK_CLICK_NOISE) {
		result = context->CheckClickNoise();
	}
	else if (context->m_threadCommand == THR_CMD_SAVE_FILE_AS) {
		result = context->SaveFileAs(context->m_saveFileAsAf, context->m_saveFileAsPath);
	}
	context->m_threadPrev = context->m_thread;
	context->m_thread = NULL;
	// IsWorkingfalseɂȂ^C~OŌĂԁB
	context->CallCommonFinish();
	_endthreadex(result ? 0 : -1);
	return result;
}

//#pragma managed(push, off)

bool TAudioFileProcess::Encode() {
	if (m_selectedEncoder->IsEnable() == false)
		return false;
	for (; size() > 0 && m_continue;) {
		TAudioFile *af = NULL;
		std::deque<TAudioFile *>::iterator current = begin();
		for (size_t i = 0; i < size(); i++) {
			af = at(i);
			if (af->m_status < 100 && af->m_status >= 0)
				break;
			current++;
			af = NULL;
		}
		if (af == NULL)
			break;

		TAudioFilterReader afr;
		afr.Open(af);
		TTempFile tempFile("AFL");
		std::wstring wtmpOutPath;
		if (m_selectedEncoder->IsNeedPostEncode()) {
			wtmpOutPath = tempFile.GetUniPath();
		}
		else {
			wtmpOutPath = af->m_mp3FilePath;
		}
		TFileWriter fOut(wtmpOutPath/*af->m_mp3FilePath*/);
		if (!m_selectedEncoder->IsNeedPostEncode()) {
			if (Prefs::data->saveID3)
				af->SaveID3Tag(&fOut);
		}

		bool good = m_selectedEncoder->Encode(af, &fOut, &afr, &af->m_format);

		fOut.Close();

		// FormClosing̃^C~Oł͂ǂR[obNInvoke@\ȂB
		if (m_continue == false)
			break;

		if (good == false) {
			af->m_status = -101;
			CallEncodeProgress(af);
			continue;
		}

		if (m_selectedEncoder->IsNeedPostEncode()) {
			m_selectedEncoder->PostEncode(tempFile.GetPath().c_str());
			TFileReader fIn2(wtmpOutPath);
			_wremove(af->m_mp3FilePath.c_str());
			TFileWriter fOut2(af->m_mp3FilePath);
			if (Prefs::data->saveID3)
				af->SaveID3Tag(&fOut2);
			while (true) {
				int data = fIn2.GetChar();
				if (data == EOF)
					break;
				fOut2.PutChar(data);
			}
			fIn2.Close();
			fOut2.Close();
		}

		CallEncodeFinishOne(af);
	}
	return true;
}

//#pragma managed(pop)

void TAudioFileProcess::StartEncode() {
	if (IsWorking())
		return;
	m_threadCommand = THR_CMD_ENCODE;
	StartThread();
}

void TAudioFileProcess::StartThread() {
	if (IsWorking())
		return;
	m_continue = true;
	m_pause = false;
	ClearAllStatus();
	if (m_threadPrev != NULL) {
		CloseHandle(m_threadPrev);
		m_threadPrev = NULL;
	}
	m_thread = (HANDLE)_beginthreadex(NULL, 0, &TAudioFileProcess::ThreadMain, this, CREATE_SUSPENDED, NULL);
	SetThreadPriority(m_thread, m_priority);
	ResumeThread(m_thread);
}

void TAudioFileProcess::Stop() {
	m_continue = false;
}

void TAudioFileProcess::PauseEncode(bool pause) {
	m_pause = pause;
}

bool TAudioFileProcess::IsWorking() {
	return m_thread != NULL;
}

void TAudioFileProcess::Join() {
	Stop();
	TCriticalSectionEnter cse(&m_csCallback);
	while (IsWorking()) {
		Sleep(10);
	}
}

int TAudioFileProcess::GetThreadCommand() {
	return m_threadCommand;
}

void TAudioFileProcess::SetPriority(int priority) {
	m_priority = priority;
	if (m_thread != NULL)
		SetThreadPriority(m_thread, m_priority);
}

void TAudioFileProcess::CallEncodeProgress(TAudioFile *af) {
	TCriticalSectionEnter cse(&m_csCallback);
	if (EncodeProgress != NULL)
		EncodeProgress(af);
}

void TAudioFileProcess::CallEncodeFinishOne(TAudioFile *af) {
	TCriticalSectionEnter cse(&m_csCallback);
	if (EncodeFinishOne != NULL)
		EncodeFinishOne(af);
}

void TAudioFileProcess::CallCommonProgress(int percent) {
	TCriticalSectionEnter cse(&m_csCallback);
	if (CommonProgress != NULL)
		CommonProgress((void *)percent);
}

void TAudioFileProcess::CallCommonStartOne(TAudioFile *af) {
	TCriticalSectionEnter cse(&m_csCallback);
	if (CommonStartOne != NULL)
		CommonStartOne(af);
}

void TAudioFileProcess::CallCommonFinishOne(TAudioFile *af) {
	TCriticalSectionEnter cse(&m_csCallback);
	if (CommonFinishOne != NULL)
		CommonFinishOne(af);
}

void TAudioFileProcess::CallCommonFinish() {
	TCriticalSectionEnter cse(&m_csCallback);
	if (CommonFinish != NULL)
		CommonFinish(NULL);
}

void TAudioFileProcess::StartSaveFilesTo(const std::wstring &dirPath) {
	if (IsWorking())
		return;
	m_threadCommand = THR_CMD_SAVE_FILES_TO;
	m_saveFilesToDirPath = dirPath;
	StartThread();
}

void TAudioFileProcess::StartSaveMergeFile(const std::wstring &savePath) {
	if (IsWorking())
		return;
	m_threadCommand = THR_CMD_SAVE_MERGE_FILE;
	m_saveMergeFilePath = savePath;
	StartThread();
}

void TAudioFileProcess::StartSaveFileAs(TAudioFile *af, const std::wstring &savePath) {
	if (IsWorking())
		return;
	m_threadCommand = THR_CMD_SAVE_FILE_AS;
	m_saveFileAsPath = savePath;
	m_saveFileAsAf = af;
	StartThread();
}

void TAudioFileProcess::StartCheckMaxDecibel() {
	if (IsWorking())
		return;
	m_threadCommand = THR_CMD_CHECK_MAX_DECIBEL;
	StartThread();
}

void TAudioFileProcess::StartCheckPhaseGap() {
	if (IsWorking())
		return;
	m_threadCommand = THR_CMD_CHECK_PHASE_GAP;
	StartThread();
}

void TAudioFileProcess::StartCheckClickNoise() {
	if (IsWorking())
		return;
	m_threadCommand = THR_CMD_CHECK_CLICK_NOISE;
	StartThread();
}

#pragma managed(push, off)

bool TAudioFileProcess::SaveFilesTo(const std::wstring &dirPath) {
	if (size() == 0)
		return true;

	size_t framesSum = CalcAllDataFrames();

	int prevStatus = 0;
	long framesWrote = 0;

	for (size_t i = 0; i < size() && m_continue; i++) {
		TAudioFile *af = at(i);
		std::wstring savePath = TAudioFile::ChangeExtension(
			TAudioFile::ChangeDirectory(af->m_waveFilePath, dirPath), _T(".wav"));
		if (_wcsicmp(savePath.c_str(), af->m_waveFilePath.c_str()) == 0)
			continue;
		TAudioFilterReader afr;
		if (!afr.Open(af))
			continue;

		TWaveFileWriter wfw;
		if (wfw.Create(savePath, (PCMWAVEFORMAT *)&af->m_format.m_waveFormat, af->m_format.m_dataBytes) == false)
			continue;

		CallCommonStartOne(af);

		for (long framePos = 0; framePos < af->m_frameCount && m_continue; framePos++) {
			for (int ch = 0; ch < af->m_format.m_waveFormat.nChannels; ch++) {
				long sample;
				sample = afr.GetSample(ch, framePos);
				wfw.Write(sample);
			}
			framesWrote += 1;
			int status = (int)(100.0 * ((double)framesWrote / (double)framesSum));
			if (prevStatus != status) {
				CallCommonProgress(status);
				prevStatus = status;
			}
		}

		wfw.Close();
		CallCommonFinishOne(af);
	}
	return true;
}

bool TAudioFileProcess::SaveMergeFile(const std::wstring &savePath) {
	if (size() == 0)
		return true;
	TAudioFileFormat *format = NULL;
	for (size_t i = 0; i < size(); i++) {
		TAudioFile *af = at(i);
		if (af->m_waveFilePath == savePath)
			return false;
		if (format == NULL) {
			format = &af->m_format;
		}
		else if (format->IsCompatible(&af->m_format) == false) {
				return false;
		}
	}
	
	size_t framesSum = CalcAllDataFrames();
	size_t bytesSum = CalcAllDataBytes();

	TWaveFileWriter wfw;
	if (wfw.Create(savePath, (PCMWAVEFORMAT *)&format->m_waveFormat, bytesSum) == false)
		return false;

	int prevStatus = 0;
	size_t framesWrote = 0;

	for (size_t i = 0; i < size() && m_continue; i++) {
		TAudioFile *af = at(i);
		TAudioFilterReader afr;
		if (!afr.Open(af))
			continue;
		CallCommonStartOne(af);

		for (long framePos = 0; framePos < af->m_frameCount && m_continue; framePos++) {
			for (int ch = 0; ch < af->m_format.m_waveFormat.nChannels; ch++) {
				long sample;
				sample = afr.GetSample(ch, framePos);
				wfw.Write(sample);
			}
			framesWrote += 1;
			int status = (int)(100.0 * ((double)framesWrote / (double)framesSum));
			if (prevStatus != status) {
				CallCommonProgress(status);
				prevStatus = status;
			}
		}
		CallCommonFinishOne(af);
	}

	wfw.Close();
	
	return true;
}

bool TAudioFileProcess::SaveFileAs(TAudioFile *af, const std::wstring &savePath) {
	size_t framesSum = af->m_frameCount;
	int prevStatus = 0;
	long framesWrote = 0;
	if (_wcsicmp(savePath.c_str(), af->m_waveFilePath.c_str()) == 0)
		return false;
	TAudioFilterReader afr;
	if (!afr.Open(af))
		false;
	TWaveFileWriter wfw;
	if (wfw.Create(savePath, (PCMWAVEFORMAT *)&af->m_format.m_waveFormat, af->m_format.m_dataBytes) == false)
		false;
	CallCommonStartOne(af);
	for (long framePos = 0; framePos < af->m_frameCount && m_continue; framePos++) {
		for (int ch = 0; ch < af->m_format.m_waveFormat.nChannels; ch++) {
			long sample;
			sample = afr.GetSample(ch, framePos);
			wfw.Write(sample);
		}
		framesWrote += 1;
		int status = (int)(100.0 * ((double)framesWrote / (double)framesSum));
		if (prevStatus != status) {
			CallCommonProgress(status);
			prevStatus = status;
		}
	}
	wfw.Close();
	CallCommonFinishOne(af);
	return true;
}

size_t TAudioFileProcess::CalcAllDataBytes() {
	size_t bytesSum = 0;
	for (size_t i = 0; i < size(); i++) {
		TAudioFile *af = at(i);
		bytesSum += af->m_format.m_dataBytes;
	}
	return bytesSum;
}

size_t TAudioFileProcess::CalcAllDataFrames() {
	size_t framesSum = 0;
	for (size_t i = 0; i < size(); i++) {
		TAudioFile *af = at(i);
		framesSum += af->m_frameCount;
	}
	return framesSum;
}

bool TAudioFileProcess::CheckMaxDecibel() {
	if (size() == 0)
		return true;
	TProcessContext context(this, CalcAllDataFrames());
	for (size_t i = 0; i < size() && context.IsContinue(); i++) {
		TAudioFile *af = at(i);
		CallCommonStartOne(af);
		af->CheckMaxDecibel(&context);
		CallCommonFinishOne(af);
	}
	return true;
}

bool TAudioFileProcess::CheckPhaseGap() {
	if (size() == 0)
		return true;
	TProcessContext context(this, CalcAllDataFrames());
	for (size_t i = 0; i < size() && context.IsContinue(); i++) {
		TAudioFile *af = at(i);
		CallCommonStartOne(af);
		af->CheckPhaseGap(&context);
		CallCommonFinishOne(af);
	}
	return true;
}

bool TAudioFileProcess::CheckClickNoise() {
	if (size() == 0)
		return true;
	TProcessContext context(this, CalcAllDataFrames());
	for (size_t i = 0; i < size() && context.IsContinue(); i++) {
		TAudioFile *af = at(i);
		CallCommonStartOne(af);
		af->CheckClickNoise(&context);
		CallCommonFinishOne(af);
	}
	return true;
}

TAudioFilterReader::TAudioFilterReader() {
	m_af = NULL;
	m_wfr = NULL;
	m_gainFilter = NULL;
	m_deClickNoiseFilter = NULL;
	m_differentiateFilter = NULL;
	m_dcOffsetFilter = NULL;
	m_phaseMove = 0;
}

bool TAudioFilterReader::Open(TAudioFile *af) {
	m_af = af;
	m_wfr = new TWaveFileReader;
	if (!m_wfr->Open(m_af->m_waveFilePath, &m_af->m_format))
		return false;
	if (m_af->m_gain != 0.0)
		m_gainFilter = new TGainFilter(m_af->m_format.m_waveFormat.wBitsPerSample, m_af->m_gain);
	if ((m_af->m_phaseCmd == PHASE_CMD_AS_AVERAGE || m_af->m_phaseCmd == PHASE_CMD_MOVE) && m_af->m_phaseMove > 0 && m_af->m_format.m_waveFormat.nChannels == 2)
		m_phaseMove = m_af->m_phaseMove;
	if (m_af->m_deClickNoiseCmd == DECLICKNOISE_CMD_DECLICK || m_af->m_deClickNoiseCmd == DECLICKNOISE_CMD_NOISE)
		m_deClickNoiseFilter = new TDeClickNoiseFilter(m_wfr, &m_af->m_noiseFramePos, m_af->m_ccnFilePath, m_af->m_ccuFilePath, m_af->m_deClickNoiseCmd == DECLICKNOISE_CMD_DECLICK);
	if (m_af->m_deClickNoiseCmd == DECLICKNOISE_CMD_DY1 || m_af->m_deClickNoiseCmd == DECLICKNOISE_CMD_DY2)
		m_differentiateFilter = new TDifferentiateFilter(m_af->m_format.m_waveFormat.nChannels, m_af->m_deClickNoiseCmd == DECLICKNOISE_CMD_DY1 ? 1 : 2);
	if (m_af->m_dcOffsetLeft != 0.0 || m_af->m_dcOffsetRight != 0.0)
		m_dcOffsetFilter = new TDCOffsetFilter(m_af->m_format.m_waveFormat.wBitsPerSample, m_af->m_dcOffsetLeft, m_af->m_dcOffsetLeftDown, m_af->m_dcOffsetRight, m_af->m_dcOffsetRightDown);
	return true;
}

TAudioFilterReader::~TAudioFilterReader() {
	if (m_wfr != NULL)
		delete m_wfr;
	if (m_gainFilter != NULL)
		delete m_gainFilter;
	if (m_deClickNoiseFilter != NULL)
		delete m_deClickNoiseFilter;
	if (m_differentiateFilter != NULL)
		delete m_differentiateFilter;
	if (m_dcOffsetFilter != NULL)
		delete m_dcOffsetFilter;
}

// ӎ
// NbNmCYtB^͌framePos߂肷邱ƂɑΉĂȂB
// GetSample߂肷悤Ȍ`ŗpȂANbNmCYtB^
// ̃RXgƔrKvB
// 2010.09.02
long TAudioFilterReader::GetSample(int ch, long framePos) {
	// ʑ
	if (m_phaseMove > 0 && ch == 1)
		framePos -= m_phaseMove;

	long sample = m_wfr->GetSample(ch, framePos);

	// DCItZbg
	if (m_dcOffsetFilter != NULL) {
		sample = m_dcOffsetFilter->ChangeDCOffset(ch, sample);
	}

	// NbNmCY
	if (m_deClickNoiseFilter != NULL) {
		sample = m_deClickNoiseFilter->DeClickNoise(ch, framePos, sample);
	}
	
	// 
	if (m_gainFilter != NULL) {
		sample = m_gainFilter->ChangeGainLong(sample);
	}

	// f^
	if (m_differentiateFilter != NULL) {
		sample = m_differentiateFilter->Differentiate(ch, framePos, sample);
	}

	return sample;
}

#pragma managed(pop)

