#include "stdafx.h"
#include "TGlobal.h"
#include "TAudioFile.h"
#include "TTools.h"
#include "TBladeEncoder.h"
#include "FormBladeSetupDlg.h"
#include "FormSimpleLameSetupDlg.h"

BladeSetupValueTables::BladeSetupValueTables() {
	m_encodeMethods.push_back(TValueTable(_T("ABR"), VBR_METHOD_ABR));
	m_encodeMethods.push_back(TValueTable(_T("CBR"), VBR_METHOD_NONE));
	m_encodeMethods.push_back(TValueTable(_T("VBR"), VBR_METHOD_DEFAULT));
	//
	m_vbrQuality.push_back(TValueTable(_T("High"), 0));
	m_vbrQuality.push_back(TValueTable(_T("1"), 1));
	m_vbrQuality.push_back(TValueTable(_T("2"), 2));
	m_vbrQuality.push_back(TValueTable(_T("3"), 3));
	m_vbrQuality.push_back(TValueTable(_T("4"), 4));
	m_vbrQuality.push_back(TValueTable(_T("5"), 5));
	m_vbrQuality.push_back(TValueTable(_T("6"), 6));
	m_vbrQuality.push_back(TValueTable(_T("7"), 7));
	m_vbrQuality.push_back(TValueTable(_T("8"), 8));
	m_vbrQuality.push_back(TValueTable(_T("Low"), 9));
	//
	m_mode.push_back(TValueTable(_T("Joint Stereo"), BE_MP3_MODE_JSTEREO));
	m_mode.push_back(TValueTable(_T("Stereo"), BE_MP3_MODE_STEREO));
	m_mode.push_back(TValueTable(_T("Dual Channel"), BE_MP3_MODE_DUALCHANNEL));
	m_mode.push_back(TValueTable(_T("Mono"), BE_MP3_MODE_MONO));
	//
	m_bitrate.push_back(TValueTable(_T("32"), 32));
	m_bitrate.push_back(TValueTable(_T("40"), 40));
	m_bitrate.push_back(TValueTable(_T("48"), 48));
	m_bitrate.push_back(TValueTable(_T("56"), 56));
	m_bitrate.push_back(TValueTable(_T("64"), 64));
	m_bitrate.push_back(TValueTable(_T("80"), 80));
	m_bitrate.push_back(TValueTable(_T("96"), 96));
	m_bitrate.push_back(TValueTable(_T("112"), 112));
	m_bitrate.push_back(TValueTable(_T("128"), 128));
	m_bitrate.push_back(TValueTable(_T("160"), 160));
	m_bitrate.push_back(TValueTable(_T("192"), 192));
	m_bitrate.push_back(TValueTable(_T("224"), 224));
	m_bitrate.push_back(TValueTable(_T("256"), 256));
	m_bitrate.push_back(TValueTable(_T("320"), 320));
}

void TBladeEncoder::Setup(HWND window) {
	BE_CONFIG beConfig = m_beConfig;
	Form^ dlg;
	if (Prefs::data->simpleLame) {
		AudioFixLite::FormSimpleLameSetupDlg^ d = gcnew AudioFixLite::FormSimpleLameSetupDlg();
		d->m_beConfig = &beConfig;
		d->m_valueTables = &m_valueTables;
		dlg = d;
	}
	else {
		AudioFixLite::FormBladeSetupDlg^ d = gcnew AudioFixLite::FormBladeSetupDlg();
		d->m_beConfig = &beConfig;
		d->m_valueTables = &m_valueTables;
		dlg = d;
	}
	if (dlg->ShowDialog() != DialogResult::OK) {
		return;
	}
	m_beConfig = beConfig;
	ValidateConfig();
	m_modify = true;
}

void TBladeEncoder::ValidateConfig() {
	if (m_beConfig.format.LHV1.nVbrMethod == VBR_METHOD_NONE) {
		m_beConfig.format.LHV1.dwVbrAbr_bps = m_beConfig.format.LHV1.dwBitrate * 1000;
		m_beConfig.format.LHV1.dwMaxBitrate = m_beConfig.format.LHV1.dwBitrate;
		m_beConfig.format.LHV1.bWriteVBRHeader = false;
	}
	else {
		m_beConfig.format.LHV1.bWriteVBRHeader = true;
	}
	if (m_beConfig.format.LHV1.dwMaxBitrate < m_beConfig.format.LHV1.dwBitrate) {
		m_beConfig.format.LHV1.dwMaxBitrate = m_beConfig.format.LHV1.dwBitrate;
	}
}

TBladeEncoder::TBladeEncoder() {
	m_enable = false;
	m_hDLL = NULL;
}

bool TBladeEncoder::Init(TAudioFileProcess *context) {
	TEncoderBase::Init(context);
	m_enable = false;

	m_hDLL = LoadLibrary(_T("lame_enc.dll"));
  	if (m_hDLL == NULL) {
  		return false;
  	}
	beInitStream	= (BEINITSTREAM) GetProcAddress(m_hDLL, TEXT_BEINITSTREAM);
	beEncodeChunk	= (BEENCODECHUNK) GetProcAddress(m_hDLL, TEXT_BEENCODECHUNK);
	beDeinitStream	= (BEDEINITSTREAM) GetProcAddress(m_hDLL, TEXT_BEDEINITSTREAM);
	beCloseStream	= (BECLOSESTREAM) GetProcAddress(m_hDLL, TEXT_BECLOSESTREAM);
	beVersion		= (BEVERSION) GetProcAddress(m_hDLL, TEXT_BEVERSION);
	beWriteVBRHeader= (BEWRITEVBRHEADER) GetProcAddress(m_hDLL,TEXT_BEWRITEVBRHEADER);
	beWriteInfoTag  = (BEWRITEINFOTAG) GetProcAddress(m_hDLL,TEXT_BEWRITEINFOTAG);
	if(!beInitStream || !beEncodeChunk || !beDeinitStream || !beCloseStream || !beVersion || !beWriteVBRHeader) {
		FreeLibrary(m_hDLL);
		m_hDLL = NULL;
		return false;
	}

	memset(&m_beConfig, 0, sizeof(m_beConfig));
	m_beConfig.dwConfig = BE_CONFIG_LAME;
//	m_beConfig.format.mp3.wBitrate = 128;
	m_beConfig.format.LHV1.dwStructVersion	= 1;
	m_beConfig.format.LHV1.dwStructSize		= sizeof(m_beConfig);		
	m_beConfig.format.LHV1.dwSampleRate		= 44100;				// INPUT FREQUENCY
	m_beConfig.format.LHV1.dwReSampleRate		= 0;					// DON"T RESAMPLE
	m_beConfig.format.LHV1.nMode				= BE_MP3_MODE_JSTEREO;	// OUTPUT IN STREO
//	m_beConfig.format.LHV1.dwBitrate			= 80;					// MINIMUM BIT RATE
	m_beConfig.format.LHV1.dwBitrate			= 128;					// MINIMUM BIT RATE
	m_beConfig.format.LHV1.dwMaxBitrate = 128;
	m_beConfig.format.LHV1.nPreset			= LQP_NOPRESET;		// QUALITY PRESET SETTING
//	m_beConfig.format.LHV1.nPreset			= LQP_ABR;		// QUALITY PRESET SETTING
//	m_beConfig.format.LHV1.nPreset			= LQP_CBR;		// QUALITY PRESET SETTING
	m_beConfig.format.LHV1.nVbrMethod =  VBR_METHOD_NONE;
	m_beConfig.format.LHV1.bEnableVBR = true;
	m_beConfig.format.LHV1.dwMpegVersion		= MPEG1;				// MPEG VERSION (I or II)
	m_beConfig.format.LHV1.dwPsyModel			= 0;					// USE DEFAULT PSYCHOACOUSTIC MODEL 
	m_beConfig.format.LHV1.dwEmphasis			= 0;					// NO EMPHASIS TURNED ON
	m_beConfig.format.LHV1.bOriginal			= true;					// SET ORIGINAL FLAG
	m_beConfig.format.LHV1.bWriteVBRHeader	= false;					// Write INFO tag
	m_beConfig.format.LHV1.bNoRes				= true;					// No Bit resorvoir
	m_beConfig.format.LHV1.dwVbrAbr_bps = 128 * 1000;
	m_enable = true;

	Load();

	return m_enable;
}

#pragma managed(push, off)

bool TBladeEncoder::Encode(TAudioFile *af, TFileWriter *fOut, TAudioFilterReader *fIn, TAudioFileFormat *format) {
	if (IsEnable() == false)
		return false;
	// ͂WAVEf[^16bitXeIƂÔ悤B
	if (format->m_waveFormat.wBitsPerSample != 16 || format->m_waveFormat.nChannels > 2)
		return false;

	BE_CONFIG beConfig = m_beConfig;
	beConfig.format.LHV1.dwSampleRate = format->m_waveFormat.nSamplesPerSec;
	if (format->m_waveFormat.nChannels == 1)
		beConfig.format.LHV1.nMode = BE_MP3_MODE_MONO;

	DWORD waveBufferWords = 0;
	DWORD mp3BufferBytes = 0;
	m_hbeStream = 0;
	BE_ERR err = beInitStream(&beConfig, &waveBufferWords, &mp3BufferBytes, &m_hbeStream);
	if (err != BE_ERR_SUCCESSFUL) {
		return false;
	}
	DWORD waveBufferFrames = waveBufferWords / format->m_waveFormat.nChannels;
	PBYTE mp3Buffer = new BYTE[mp3BufferBytes];
	PSHORT waveBuffer = new SHORT[waveBufferWords];

	DWORD writeBytes;
	DWORD rest = af->m_frameCount;
	long framePos = 0;
	while (rest > 0 && m_context->m_continue) {
		if (m_context->m_pause) {
			Sleep(100);
			continue;
		}
		int status = (int)(100.0 * ((double)af->m_frameCount - (double)rest) / (double)af->m_frameCount);
		if (af->m_status != status) {
			af->m_status = status;
			m_context->CallEncodeProgress(af);
		}
		size_t readFrames = rest;
		if (rest > waveBufferFrames)
			readFrames = waveBufferFrames;
		size_t p = 0;
		for (size_t i = 0; i < readFrames; i++) {
			for (int ch = 0; ch < format->m_waveFormat.nChannels; ch++) {
				waveBuffer[p] = (SHORT)fIn->GetSample(ch, framePos);
				p++;
			}
			framePos++;
		}
		
		size_t encodeWords = readFrames * format->m_waveFormat.nChannels;
		if (format->m_waveFormat.nChannels == 2 && beConfig.format.LHV1.nMode == BE_MP3_MODE_MONO) {
			for (size_t i = 0, j = 0; i < readFrames * 2; i += 2, j++) {
				long sample1 = SampleRawToLong((const BYTE *)&waveBuffer[i], sizeof(SHORT), false);
				long sample2 = SampleRawToLong((const BYTE *)&waveBuffer[i + 1], sizeof(SHORT), false);
				waveBuffer[j] = (SHORT)TGainFilter::MonoLong(sample1, sample2);
			}
			encodeWords /= 2;
		}

		err = beEncodeChunk(m_hbeStream, encodeWords, waveBuffer, mp3Buffer, &writeBytes);
		if (err != BE_ERR_SUCCESSFUL) {
			beCloseStream(m_hbeStream);
			return false;
		}
		fOut->Write(mp3Buffer, writeBytes);
		rest -= readFrames;
	}

	err = beDeinitStream(m_hbeStream, mp3Buffer, &writeBytes);
	if (err != BE_ERR_SUCCESSFUL) {
		beCloseStream(m_hbeStream);
		return false;
	}
	if (writeBytes) {
		fOut->Write(mp3Buffer, writeBytes);
	}
	beCloseStream(m_hbeStream);
	delete [] waveBuffer;
	delete [] mp3Buffer;
	if (m_context->m_continue) {
		af->m_status = 100;
		m_context->CallEncodeProgress(af);
	}
	return true;
}

bool TBladeEncoder::PostEncode(const char *path) {
	if (m_beConfig.format.LHV1.bWriteVBRHeader) {
		// Write the INFO Tag
		if (beWriteInfoTag)
			beWriteInfoTag(m_hbeStream, path);
		else
			beWriteVBRHeader(path);
	}
	return true;
}

#pragma managed(pop)

TBladeEncoder::~TBladeEncoder() {
	if (m_hDLL != NULL) {
		Save();
		FreeLibrary(m_hDLL);
	}
}

void TBladeEncoder::Load() {
	struct _stat status;
	if (_wstat(GetSaveFilePath().c_str(), &status) != -1) {
		FILE *fp = _wfopen(GetSaveFilePath().c_str(), _T("rb"));
		if (fp != NULL) {
			fread(&m_beConfig, sizeof(m_beConfig), 1, fp);
			fclose(fp);
			ValidateConfig();
			m_enable = true;
		}
	}
	m_modify = false;
}

void TBladeEncoder::Save() {
	if (m_modify) {
		FILE *fp = _wfopen(GetSaveFilePath().c_str(), _T("wb"));
		if (fp != NULL) {
			fwrite(&m_beConfig, sizeof(m_beConfig), 1, fp);
			fclose(fp);
		}
	}
}

std::wstring TBladeEncoder::GetSaveFilePath() {
	std::wstring s = TStringUtil::MarshalString(Preferences::GetPrefDirPath());
	s += _T("BladeSetting.dat");
	return s;
}

