#ifndef _TDECLICKNOISE_H_
#define _TDECLICKNOISE_H_

#include "TTools.h"
#include "TWaveFileReader.h"
#include "TGainFilter.h"

#pragma managed(push, off)

class TNoiseFramePos {
public:
	long m_start;
	long m_end;
	TNoiseFramePos() {
		m_start = 0;
		m_end = 0;
	}
	TNoiseFramePos(long start, long end) {
		m_start = start;
		m_end = end;
		if (m_start == m_end)
			m_end++;
	}
	long GetLength() {
		return m_end - m_start + 1;
	}
};

class TNoiseFramePosMultiCh : public std::vector<std::vector<TNoiseFramePos> > {
public:
	void Setup(int ch);
	long GetFramesSum(int ch);
	long GetFramesSum();
	class Less {
	public:
		bool operator() (TNoiseFramePos p1, TNoiseFramePos p2) {
			return p1.m_start < p2.m_start;
		}
	};
	void Sort();
	void Merge(const TNoiseFramePosMultiCh *p);
	bool Save(const std::wstring &path);
	bool Load(const std::wstring &path, int ch);
};

class TClickNoiseChecker {
public:
	int m_ch;
	long m_samplePrev;
	long m_delta1Prev;
	TAverageOfLong m_averOfDelta2;
	TAverageOfLong m_averOfSample;
	TWaveFileReader *m_wfr;
	long m_frameCount;
	TGainFilter m_gf;
	//
	double m_clickNoiseThrFix;
	int m_clickNoiseThrAvrMag1;
	int m_clickNoiseThrAvrMag2;
	int m_clickNoiseCvgThrAvrMag;
	//int m_clickNoiseCvgCount;
	int m_clickNoiseAvrCount;
	int m_clickNoiseMaxDistance;
	bool m_clickNoiseLeft;
	bool m_clickNoiseRight;
	long m_startFramePos;
	long m_endFramePos;
	//
	TClickNoiseChecker(TWaveFileReader *wfr, int ch, long frameCount, long st, long en);
	void Setup(TWaveFileReader *wfr, int ch, long frameCount, long st, long en);
	bool CheckNoise(long framePos, long sample, TNoiseFramePosMultiCh *noiseFramePos);
	long SearchStart(long framePos, long cnv, long maxDistance);
	long SearchEnd(long framePos, long cnv, long maxDistance);
	long CorrectFramePos(long f);
};

class TRepairNoise {
public:
	TNoiseFramePos m_currentNoise;
	float *m_repairBuffer;
	long m_offset;
	TRepairNoise() {
		m_repairBuffer = NULL;
	}
	~TRepairNoise() {
		if (m_repairBuffer != NULL)
			delete [] m_repairBuffer;
	}
};

class TDeClickNoiseFilter {
public:
	TWaveFileReader *m_wfr;
	TNoiseFramePosMultiCh m_noiseFramePos;
	bool m_deClick;
	int *m_index;
	std::vector<TRepairNoise *> m_repairNoise;
	TDeClickNoiseFilter(TWaveFileReader *wfr, const TNoiseFramePosMultiCh *noiseFramePos, const std::wstring &ccnFilePath, const std::wstring &ccuFilePath, bool deClick);
	~TDeClickNoiseFilter();
	long DeClickNoise(int ch, long framePos, long sample);
};

class TDifferentiateFilter {
public:
	long *m_samplePrev;
	long *m_delta1Prev;
	int m_dif;
	TDifferentiateFilter(int ch, int dif) {
		m_dif = dif;
		m_samplePrev = new long[ch];
		m_delta1Prev = new long[ch];
	}
	~TDifferentiateFilter() {
		delete [] m_samplePrev;
		delete [] m_delta1Prev;
	}
	long Differentiate(int ch, long framePos, long sample) {
		long d1 = sample - m_samplePrev[ch];
		long d2 = d1 - m_delta1Prev[ch];
		m_samplePrev[ch] = sample;
		m_delta1Prev[ch] = d1;
		if (m_dif == 1)
			return d1;
		if (m_dif == 2)
			return d2;
		return sample;
	}
};

#pragma managed(pop)

#endif
