#include "stdafx.h"
#include "TTools.h"

using namespace Runtime::InteropServices;

std::wstring TStringUtil::MarshalString(String^ s) {
	std::wstring result;
	const wchar_t* chars = (const wchar_t*)(Marshal::StringToHGlobalUni(s)).ToPointer();
	result = chars;
	Marshal::FreeHGlobal(IntPtr((void*)chars));
	return result;
}

int TStringUtil::Split(const std::string &line, const std::string &dlmt, std::vector<std::string> *slist) {
	slist->clear();
	char *buffer = new char [line.length() + 1];
	strcpy(buffer, line.c_str());
	char *d = strtok(buffer, dlmt.c_str());
	while (d != NULL) {
		slist->push_back(d);
		d = strtok(NULL, dlmt.c_str());
	}
	delete [] buffer;
	return (int)slist->size();
}

bool TStringUtil::IsDigit(const std::string &s) {
	if (s.length() == 0)
		return false;
	for (int i = 0; i < (int)s.length(); i++) {
		if (isdigit(s[i]) || s[i] == ',') {
		}
		else {
			return false;
		}
	}
	return true;
}

int TStringUtil::ToInteger(const std::string &s) {
	std::vector<std::string> slist;
	Split(s, ",", &slist);
	std::string ns;
	for (int i = 0; i < (int)slist.size(); i++) {
		ns += slist[i];
	}
	return atoi(ns.c_str());
}

int TStringUtil::ToInteger(String^ s) {
	return ToInteger(ToAnsi(MarshalString(s)));
}

std::string TStringUtil::ToAnsi(const std::wstring &s) {
	if (s.length() == 0)
		return "";
	int sizeOfBuffer = s.length() * 4 + 16;
	char *buffer = new char[sizeOfBuffer];
	WideCharToMultiByte(CP_ACP, 0, s.c_str(), -1, buffer, sizeOfBuffer, NULL, NULL);
	std::string result = buffer;
	delete [] buffer;
	return result;
}

std::wstring TStringUtil::ToUnicode(const std::string &s) {
	if (s.length() == 0)
		return L"";
	int sizeOfBuffer = s.length() * 4 + 16;
	wchar_t *buffer = new wchar_t[sizeOfBuffer];
	MultiByteToWideChar(CP_ACP, 0, s.c_str(), -1, buffer, sizeOfBuffer);
	std::wstring result = buffer;
	delete [] buffer;
	return result;
}

void TGenericCallbackGlue::GlueFunction(void *param) {
	if (m_enable && m_control->Enabled)
		m_control->Invoke(m_callback, gcnew TGenericCallbackParam(param));
}

TGenericCallbackGlue::TGenericCallbackGlue(System::Windows::Forms::Control^ control, CallbackDelegate^ callback) {
	m_control = control;
	m_glueDelegate = gcnew GlueDelegate(this, &TGenericCallbackGlue::GlueFunction);
	m_gch = GCHandle::Alloc(m_glueDelegate);
	m_glue = static_cast<TGENERIC_CALLBACK>(
		Marshal::GetFunctionPointerForDelegate(m_glueDelegate).ToPointer());
	m_callback = callback;
	m_enable = true;
}

/*
TPrivateProfile::TPrivateProfile(const std::wstring &filePath) {
	m_filePath = filePath;
}

void TPrivateProfile::PutString(const std::wstring &section, const std::wstring &key, const std::wstring &value) {
	WritePrivateProfileString(section.c_str(), key.c_str(), value.c_str(), m_filePath.c_str());
}

void TPrivateProfile::PutInteger(const std::wstring &section, const std::wstring &key, int value) {
	wchar_t s[64];
	swprintf(s, _T("%d"), value);
	WritePrivateProfileString(section.c_str(), key.c_str(), s, m_filePath.c_str());
}

void TPrivateProfile::PutBool(const std::wstring &section, const std::wstring &key, bool value) {
	WritePrivateProfileString(section.c_str(), key.c_str(), value ? _T("true") : _T("false"), m_filePath.c_str());
}

std::wstring TPrivateProfile::GetString(const std::wstring &section, const std::wstring &key, const std::wstring &defaultValue) {
	wchar_t s[1024];
	GetPrivateProfileString(section.c_str(), key.c_str(), 
		defaultValue.c_str(), s, sizeof(s), m_filePath.c_str());
	return s;
}

int TPrivateProfile::GetInteger(const std::wstring &section, const std::wstring &key, int defaultValue) {
	return (int)GetPrivateProfileInt(section.c_str(), key.c_str(), defaultValue, m_filePath.c_str());
}

bool TPrivateProfile::GetBool(const std::wstring &section, const std::wstring &key, bool defaultValue) {
	wchar_t s[64];
	GetPrivateProfileString(section.c_str(), key.c_str(), 
		defaultValue ? _T("true") : _T("false"), s, sizeof(s), m_filePath.c_str());
	return (
		_wcsicmp(s, _T("true")) == 0 ||
		_wcsicmp(s, _T("ok")) == 0 ||
		_wcsicmp(s, _T("yes")) == 0 ||
		_wcsicmp(s, _T("on")) == 0 ||
		_wcsicmp(s, _T("good")) == 0 ||
		_wcsicmp(s, _T("t")) == 0 ||
		_wcsicmp(s, _T("y")) == 0 ||
		_wcsicmp(s, _T("1")) == 0
	);
}
*/
